NewHollandBB940A = {};

function NewHollandBB940A.prerequisitesPresent(specializations)
    return true;
end;

function NewHollandBB940A:load(xmlFile)

    self.brake = SpecializationUtil.callSpecializationsFunction("brake");
	self.releaseBrake = SpecializationUtil.callSpecializationsFunction("releaseBrake");
	self.moveBalesOut = SpecializationUtil.callSpecializationsFunction("moveBalesOut");
	
	self.brakeLight = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.brakeLight#index"));
	setVisibility(self.brakeLight, false);
	
	self.getTimeFromLevel = NewHollandBB940A.getTimeFromLevel;
    self.moveBales = SpecializationUtil.callSpecializationsFunction("moveBales");
    self.moveBale = SpecializationUtil.callSpecializationsFunction("moveBale");
    self.allowFillType = NewHollandBB940A.allowFillType;

    self.fillScale = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.fillScale#value"), 1);

    local firstBaleMarker = getXMLFloat(xmlFile, "vehicle.baleAnimation#firstBaleMarker");
    if firstBaleMarker ~= nil then
        local baleAnimCurve = AnimCurve:new(linearInterpolatorN);
        local keyI = 0;
        while true do
            local key = string.format("vehicle.baleAnimation.key(%d)", keyI);
            local t = getXMLFloat(xmlFile, key.."#time");
            local x,y,z = Utils.getVectorFromString(getXMLString(xmlFile, key.."#pos"));
            if x == nil or y == nil or z == nil then
                break;
            end;
            local rx, ry, rz = Utils.getVectorFromString(getXMLString(xmlFile, key.."#rot"));
            rx = math.rad(Utils.getNoNil(rx, 0));
            ry = math.rad(Utils.getNoNil(ry, 0));
            rz = math.rad(Utils.getNoNil(rz, 0));
            baleAnimCurve:addKeyframe({ v={x, y, z, rx, ry, rz}, time = t});
            keyI = keyI +1;
        end;
        if keyI > 0 then
            self.baleAnimCurve = baleAnimCurve;
            self.baleAnimRoot = Utils.getNoNil(Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.baleAnimation#node")), self.components[1].node);
            self.firstBaleMarker = firstBaleMarker;
            self.baleTypes = {};
            local i = 0;
            while true do
                local key = string.format("vehicle.baleTypes.baleType(%d)", i);
                local t = getXMLString(xmlFile, key.."#fruitType");
                local filename = getXMLString(xmlFile, key.."#filename");
                if t==nil or filename==nil then
                    break;
                end;

                local entry = {};
                entry.filename = filename;
                local desc = FruitUtil.fruitTypes[t];
                if desc ~= nil then
                    self.baleTypes[desc.index] = entry;

                    if self.defaultBaleType == nil then
                        self.defaultBaleType = entry;
                    end;
                end;
                i = i +1;
            end;
            if self.defaultBaleType == nil then
                self.baleTypes = nil;
            end;
        end;
    end;
	
	local rotationPartsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rotationParts#count"), 0);	
	self.rotationParts = {};
	for i=1, rotationPartsCount do
		local rotationPartName = string.format("vehicle.rotationParts.rotationPart%d", i);		
		self.rotationParts[i] = {};		
		self.rotationParts[i].node = Utils.indexToObject(self.components, getXMLString(xmlFile, rotationPartName .. "#node"));
		self.rotationParts[i].speed = Utils.getNoNil(getXMLInt(xmlFile, rotationPartName .. "#speed"), 1);
	end;
	
	local workSound = getXMLString(xmlFile, "vehicle.workSound#file");
    if workSound ~= nil and workSound ~= "" then
        workSound = Utils.getFilename(workSound, self.baseDirectory);
        self.workSound = createSample("workSound");
        self.workSoundEnabled = false;
        loadSample(self.workSound, workSound, false);
        self.workSoundPitchOffset = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workSound#pitchOffset"), 1);
        self.workSoundVolume = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.workSound#volume"), 1);
    end;
	
	local particleSystemCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.particleSystems#count"), 0);
	self.particleSystems = {};
    for i=1, particleSystemCount do
        local namei = string.format("vehicle.particleSystems.particleSystem%d", i);
		local nodei = Utils.indexToObject(self.components, getXMLString(xmlFile, namei .. "#index"));
        Utils.loadParticleSystem(xmlFile, self.particleSystems, namei, nodei, false, nil, self.baseDirectory)			
    end;
	self.particleSystemActive = false;
	
	self.counterPoxX = 1-0.168;
    self.counterPoxY = 1-0.420; 
    self.counterWidth = 0.180;
    self.counterHeight = 0.320;
	self.counterpath = Utils.getFilename("counter.png", self.baseDirectory);
    self.hudPDAOverlay = Overlay:new("counterOverlay", self.counterpath, self.counterPoxX, self.counterPoxY, self.counterWidth, self.counterHeight);

    self.baleMoveLastTime = 0;
    self.bales = {};
    self.wasToFast = false;
    self.isTurnedOn = false;
	self.baleCount = 0;
	self.printWait = false;
	self.transportMode = true;
	self.moveBalesOutside = false;
	
	self.Speed.NewHollandPickUp = 10;
	self.Go.NewHollandPickUp = true;
	self.Done.NewHollandPickUp = true;
	self.Speed.NewHollandTransport = 10;
	self.Go.NewHollandTransport = true;
	self.Done.NewHollandTransport = true;
end;

function NewHollandBB940A:delete() 
	if self.workSound ~= nil then
        delete(self.workSound);
    end;
	Utils.deleteParticleSystem(self.particleSystems);
end;

function NewHollandBB940A:mouseEvent(posX, posY, isDown, isUp, button)
end;

function NewHollandBB940A:keyEvent(unicode, sym, modifier, isDown)
end;

function NewHollandBB940A:getSaveAttributesAndNodes(nodeIdent)
	local transport = "false";
	if self.CheckDone.NewHollandTransport then
		transport = "true";
	end;
	local pickUpDown = "true";
	if self.CheckDone.NewHollandPickUp then
		pickUpDown = "false";
	end;
	local bale1Time = 0;
	local bale2Time = 0;
	local bale3Time = 0;
	local bale1 = self.bales[1];
	local bale2 = self.bales[2];
	local bale3 = self.bales[3];
	if bale1 ~= nil and bale1.time ~= nil then
		bale1Time = bale1.time;
	end;
	if bale2 ~= nil and bale2.time ~= nil then
		bale2Time = bale2.time;
	end;
	if bale3 ~= nil and bale3.time ~= nil then
		bale3Time = bale3.time;
	end;
	local attributes = 'transport="'..transport..'" pickUpDown="'..pickUpDown..'" baleCount="'..self.baleCount..'" bales="'..bale1Time..' '..bale2Time..' '..bale3Time..'"';
	return attributes, nil;
end;

function NewHollandBB940A:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
    local transport = getXMLString(xmlFile, key.."#transport");
	local pickUpDown = getXMLString(xmlFile, key.."#pickUpDown");
	local baleCount = getXMLFloat(xmlFile, key.."#baleCount");
	if not resetVehicles then
		if transport == "true" then
			self.transportMode = true;
			self.Speed.NewHollandTransport = 10;
			self.Go.NewHollandTransport = true;
			self.Done.NewHollandTransport = true;
			self.moveBalesOutside = true;
		else
			self.transportMode = false;
			self.Speed.NewHollandTransport = 10;
			self.Go.NewHollandTransport = false;
			self.Done.NewHollandTransport = true;
		end;
		if pickUpDown == "false" then
			self.Speed.NewHollandPickUp = 10;
			self.Go.NewHollandPickUp = true;
			self.Done.NewHollandPickUp = true;
		else
			self.Speed.NewHollandPickUp = 10;
			self.Go.NewHollandPickUp = false;
			self.Done.NewHollandPickUp = true;
		end;
		if baleCount ~= nil then
			self.baleCount = baleCount;
		end;
		local bale1Time, bale2Time, bale3Time = Utils.getVectorFromString(getXMLString(xmlFile, key.."#bales"));
		if bale1Time ~= nil and bale1Time ~= 0 then
			if self.baleAnimCurve ~= nil and self.baleTypes ~= nil then
				local baleType = self.baleTypes[usedFruitType];
				if baleType == nil then
					baleType = self.defaultBaleType;
				end;
				local baleRoot = Utils.loadSharedI3DFile(baleType.filename, self.baseDirectory);
				local baleId = getChildAt(baleRoot, 0);
				setRigidBodyType(baleId, "None");
				link(self.baleAnimRoot, baleId);
				delete(baleRoot);
				local bale = {};
				bale.id = baleId;
				bale.time = bale1Time;
				bale.filename = Utils.getFilename(baleType.filename, self.baseDirectory);
				table.insert(self.bales, bale);
				local v = self.baleAnimCurve:get(bale.time);
				setTranslation(bale.id, v[1], v[2], v[3]);
				setRotation(bale.id, v[4], v[5], v[6]);
				bale.lastX, bale.lastY, bale.lastZ = getWorldTranslation(bale.id);
			end;
		end;
		if bale2Time ~= nil and bale2Time ~= 0 then
			if self.baleAnimCurve ~= nil and self.baleTypes ~= nil then
				local baleType = self.baleTypes[usedFruitType];
				if baleType == nil then
					baleType = self.defaultBaleType;
				end;
				local baleRoot = Utils.loadSharedI3DFile(baleType.filename, self.baseDirectory);
				local baleId = getChildAt(baleRoot, 0);
				setRigidBodyType(baleId, "None");
				link(self.baleAnimRoot, baleId);
				delete(baleRoot);
				local bale = {};
				bale.id = baleId;
				bale.time = bale2Time;
				bale.filename = Utils.getFilename(baleType.filename, self.baseDirectory);
				table.insert(self.bales, bale);
				local v = self.baleAnimCurve:get(bale.time);
				setTranslation(bale.id, v[1], v[2], v[3]);
				setRotation(bale.id, v[4], v[5], v[6]);
				bale.lastX, bale.lastY, bale.lastZ = getWorldTranslation(bale.id);
			end;
		end;
		if bale3Time ~= nil and bale3Time ~= 0 then
			if self.baleAnimCurve ~= nil and self.baleTypes ~= nil then
				local baleType = self.baleTypes[usedFruitType];
				if baleType == nil then
					baleType = self.defaultBaleType;
				end;
				local baleRoot = Utils.loadSharedI3DFile(baleType.filename, self.baseDirectory);
				local baleId = getChildAt(baleRoot, 0);
				setRigidBodyType(baleId, "None");
				link(self.baleAnimRoot, baleId);
				delete(baleRoot);
				local bale = {};
				bale.id = baleId;
				bale.time = bale3Time;
				bale.filename = Utils.getFilename(baleType.filename, self.baseDirectory);
				table.insert(self.bales, bale);
				local v = self.baleAnimCurve:get(bale.time);
				setTranslation(bale.id, v[1], v[2], v[3]);
				setRotation(bale.id, v[4], v[5], v[6]);
				bale.lastX, bale.lastY, bale.lastZ = getWorldTranslation(bale.id);
			end;
		end;
	end;
    return BaseMission.VEHICLE_LOAD_OK;
end;

function NewHollandBB940A:update(dt)

	self:anim("NewHollandSupport", false);
	self:anim("NewHollandPickUp", false);
	self:anim("NewHollandTransport", false);
	
	for i=1, table.getn(self.componentJoints) do 	
		local joint = self.componentJoints[i];
		setJointFrame(joint.jointIndex, 0,joint.jointNode);
	end;

	if self:getIsActiveForInput() then
        if InputBinding.hasEvent(InputBinding.NewHollandTransport) and self.isTurnedOn == false then
			self.transportMode = not self.transportMode;
		end;
		if self.transportMode then
			if self.moveBalesOutside == true then
				self.printWait = false;
				self.Speed.NewHollandTransport = 1;
				self.Go.NewHollandTransport = true;
				self.Done.NewHollandTransport = true;
			else
				self:moveBalesOut(dt);
				self.printWait = true;
			end;
			self.fillLevel = 0;
			self.fillType = FruitUtil.FRUITTYPE_UNKNOWN;
			self:setFillLevel(self.fillLevel, self.fillType);
		else
			self.moveBalesOutside = false;
			self.printWait = false;
			self.Go.NewHollandTransport = false;
			self.Done.NewHollandTransport = true;
        end;
		if self.currentPosition.NewHollandTransport < 200 then
			if InputBinding.hasEvent(InputBinding.NewHollandEinschalten) then
				self.isTurnedOn = not self.isTurnedOn;
			end;
			if InputBinding.hasEvent(InputBinding.NewHollandPickup) then
				self.Speed.NewHollandPickUp = 1;
				self.Go.NewHollandPickUp = not self.Go.NewHollandPickUp;
				self.Done.NewHollandPickUp = true;
			end;
		else
			self.isTurnedOn = false;
			self.Go.NewHollandPickUp = true;
			self.Done.NewHollandPickUp = true;
		end;
    end;
	if self.currentPosition.NewHollandPickUp < 400 then
		setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, math.rad(-23), math.rad(23));
	else
		setJointRotationLimit(self.componentJoints[2].jointIndex, 0, true, math.rad(0), math.rad(0));
	end;
    self.wasToFast = false;
    if self:getIsActive() then
		if self.isTurnedOn then
			for k, rotPart in pairs(self.rotationParts) do
				rotate(self.rotationParts[k].node, self.rotationParts[k].speed*dt, 0, 0);
			end;
			if not self.workSoundEnabled and self:getIsActiveForSound() then
				setSamplePitch(self.workSound, self.workSoundPitchOffset);
				playSample(self.workSound, 0, self.workSoundVolume, 0);
				self.workSoundEnabled = true;
			end;
			if self.currentPosition.NewHollandTransport < 600 and self.currentPosition.NewHollandPickUp < 100 then
				local toFast = self:doCheckSpeedLimit() and self.lastSpeed*3600 > 22;
				if not toFast then
					local totalArea =0;
					local usedFruitType = FruitUtil.FRUITTYPE_UNKNOWN;
					for k, cuttingArea in pairs(self.cuttingAreas) do
						local x,y,z = getWorldTranslation(cuttingArea.start);
						local x1,y1,z1 = getWorldTranslation(cuttingArea.width);
						local x2,y2,z2 = getWorldTranslation(cuttingArea.height);
						for fruitType,v in pairs(self.fillTypes) do
							if fruitType ~= FruitUtil.FRUITTYPE_UNKNOWN then
								local area = Utils.updateFruitWindrowArea(fruitType, x, z, x1, z1, x2, z2, 0)*g_currentMission.windrowCutLongRatio;
								area = area + Utils.updateFruitCutLongArea(fruitType, x, z, x1, z1, x2, z2, 0);
								if area > 0 then
									totalArea = totalArea+area;
									usedFruitType = fruitType;
								end;
							end;
						end;
					end;
					if totalArea > 0 then
						local literPerPixel = 4*8000/1200 / 6 / (2*2) *12/4 / 4;
						local deltaLevel = totalArea * literPerPixel * self.fillScale;
						local deltaTime = self:getTimeFromLevel(deltaLevel);
						self:moveBales(deltaTime);
						local oldFillLevel = self.fillLevel;
						self:setFillLevel(self.fillLevel+deltaLevel, usedFruitType);
						if self.fillLevel == self.capacity then
							local restDeltaFillLevel = deltaLevel - (self.fillLevel-oldFillLevel)
							self:setFillLevel(restDeltaFillLevel, usedFruitType);
							if self.baleAnimCurve ~= nil and self.baleTypes ~= nil then
								local baleType = self.baleTypes[usedFruitType];
								if baleType == nil then
									baleType = self.defaultBaleType;
								end;
								local baleRoot = Utils.loadSharedI3DFile(baleType.filename, self.baseDirectory);
								local baleId = getChildAt(baleRoot, 0);
								setRigidBodyType(baleId, "None");
								link(self.baleAnimRoot, baleId);
								delete(baleRoot);
								local bale = {};
								bale.id = baleId;
								bale.time = 0;
								bale.filename = Utils.getFilename(baleType.filename, self.baseDirectory);
								table.insert(self.bales, bale);
								self:moveBale(table.getn(self.bales), self:getTimeFromLevel(restDeltaFillLevel));
							end;
						end;
						Utils.setEmittingState(self.particleSystems, true);
					else
						Utils.setEmittingState(self.particleSystems, false);
					end;
				end;
				self.wasToFast = toFast;
			else
				Utils.setEmittingState(self.particleSystems, false);
			end;
		end;
    end;
	if not self.isTurnedOn and self.workSoundEnabled then
		stopSample(self.workSound);
		self.workSoundEnabled = false;
	end;
end;

function NewHollandBB940A:draw()
	if self.currentPosition.NewHollandTransport < 200 then
		self.hudPDAOverlay:render();
		setTextBold(true);
		setTextColor(0, 0, 0, 1.0);
		local textX = 0.118;
		if self.baleCount > 999999 then
			textX = 0.058;
		elseif self.baleCount > 99999 then
			textX = 0.068;
		elseif self.baleCount > 9999 then
			textX = 0.078;
		elseif self.baleCount > 999 then
			textX = 0.088;
		elseif self.baleCount > 99 then
			textX = 0.098;
		elseif self.baleCount > 9 then
			textX = 0.108;
		end;
		renderText(self.counterPoxX+textX, self.counterPoxY+0.27, 0.029, string.format(self.baleCount));
		if self.wasToFast then
			g_currentMission:addWarning(g_i18n:getText("Dont_drive_to_fast") .. "\n" .. string.format(g_i18n:getText("Cruise_control_levelN"), "2", InputBinding.getButtonKeyName(InputBinding.SPEED_LEVEL2)), 0.07+0.022, 0.019+0.029);
		end;
		if self.isTurnedOn then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("NewHollandBB940A_4"), self.typeDesc), InputBinding.NewHollandEinschalten);
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("NewHollandBB940A_7"), self.typeDesc), InputBinding.NewHollandTransport);
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("NewHollandBB940A_3"), self.typeDesc), InputBinding.NewHollandEinschalten);
		end;
		if not self.CheckDone.NewHollandPickUp then
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("NewHollandBB940A_6"), self.typeDesc), InputBinding.NewHollandPickup);
		else
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("NewHollandBB940A_5"), self.typeDesc), InputBinding.NewHollandPickup);
		end;
	else
		g_currentMission:addHelpButtonText(string.format(g_i18n:getText("NewHollandBB940A_8"), self.typeDesc), InputBinding.NewHollandTransport);
	end;
	if self.printWait then
		g_currentMission:addWarning(g_i18n:getText("BaleWarning"), 0.040, 0.035);
	end;
end;

function NewHollandBB940A:onAttach(attacherVehicle)
	self.attacherVehicle = attacherVehicle;
	self.Go.NewHollandSupport = true;
	self.Done.NewHollandSupport = true;
end

function NewHollandBB940A:onDetach()
    if self.deactivateOnDetach then
        NewHollandBB940A.onDeactivate(self);
    end;
	setVisibility(self.brakeLight, false);
	self.Go.NewHollandSupport = false;
	self.Done.NewHollandSupport = true;
end;

function NewHollandBB940A:onLeave()
    if self.deactivateOnLeave then
        NewHollandBB940A.onDeactivate(self);
    else
        NewHollandBB940A.onDeactivateSounds(self);
    end;
end;

function NewHollandBB940A:onDeactivate()
    self.wasToFast = false;
    self.isTurnedOn = false;
	Utils.setEmittingState(self.particleSystems, false);
    NewHollandBB940A.onDeactivateSounds(self)
end;

function NewHollandBB940A:onDeactivateSounds()
    if self.workSoundEnabled then
        stopSample(self.workSound);
        self.workSoundEnabled = false;
    end;
end;

function NewHollandBB940A:getTimeFromLevel(level)
    if self.firstBaleMarker ~= nil then
        return level / self.capacity * self.firstBaleMarker;
    end;
    return 0;
end;

function NewHollandBB940A:moveBales(dt)
    for i=table.getn(self.bales), 1, -1 do
        self:moveBale(i, dt);
    end;
    self.baleMoveLastTime = self.time
end;

function NewHollandBB940A:moveBale(i, dt)
    local bale = self.bales[i];
    bale.time = bale.time + dt;
    local v = self.baleAnimCurve:get(bale.time);
    setTranslation(bale.id, v[1], v[2], v[3]);
    setRotation(bale.id, v[4], v[5], v[6]);
    if bale.time >= 1 then
        local deltaRealTime = (self.time - self.baleMoveLastTime)/1000;
        local lx, ly, lz = bale.lastX, bale.lastY, bale.lastZ;
        local x,y,z = getWorldTranslation(bale.id);
        local rx,ry,rz = getWorldRotation(bale.id);
        link(getRootNode(), bale.id);
        g_currentMission:addItemToSave(bale.filename, bale.id, 0);
        setTranslation(bale.id, x, y, z);
        setRotation(bale.id, rx, ry, rz);
        setRigidBodyType(bale.id, "Dynamic");
        setLinearVelocity(bale.id, (x-lx)/deltaRealTime, (y-ly)/deltaRealTime, (z-lz)/deltaRealTime);
        table.remove(self.bales, i);
        if g_currentMission.baleCount ~= nil then
            g_currentMission.baleCount = g_currentMission.baleCount + 1;
        end;
		self.baleCount = self.baleCount + 1;
    else
        bale.lastX, bale.lastY, bale.lastZ = getWorldTranslation(bale.id);
    end;
end;

function NewHollandBB940A:moveBalesOut(dt)
	local done = false;
	local drive = true;
	for i=1, table.getn(self.bales) do
		local bale = self.bales[i];
		if bale ~= nil then
			bale.time = bale.time + dt/10000;
			local v = self.baleAnimCurve:get(bale.time);
			setTranslation(bale.id, v[1], v[2], v[3]);
			setRotation(bale.id, v[4], v[5], v[6]);
			if bale.time >= 1 then
				local deltaRealTime = (self.time - self.baleMoveLastTime)/1000;
				local lx, ly, lz = bale.lastX, bale.lastY, bale.lastZ;
				local x,y,z = getWorldTranslation(bale.id);
				local rx,ry,rz = getWorldRotation(bale.id);
				link(getRootNode(), bale.id);
				g_currentMission:addItemToSave(bale.filename, bale.id, 0);
				setTranslation(bale.id, x, y, z);
				setRotation(bale.id, rx, ry, rz);
				setRigidBodyType(bale.id, "Dynamic");
				setLinearVelocity(bale.id, (x-lx)/deltaRealTime, (y-ly)/deltaRealTime, (z-lz)/deltaRealTime);
				table.remove(self.bales, i);
				if g_currentMission.baleCount ~= nil then
					g_currentMission.baleCount = g_currentMission.baleCount + 1;
				end;
				self.baleCount = self.baleCount + 1;
			end;
		else
			drive = false;
		end;
	end;
	if table.getn(self.bales) == 0 then
		done = true;
		drive = false;
	end;
	if self.attacherVehicle ~= nil then
		local kmh = self.attacherVehicle.lastSpeed*self.attacherVehicle.speedDisplayScale*3600;
		if drive then
			if kmh < 2 then
				self.attacherVehicle.motor:setSpeedLevel(1, true);
			else
				self.attacherVehicle.motor:setSpeedLevel(0, false);
			end;
		else
			self.attacherVehicle.motor:setSpeedLevel(0, false);
		end;
	end;
	self.moveBalesOutside = done;
end;
		
function NewHollandBB940A:allowFillType(fillType)
    return self.fillTypes[fillType] == true;
end;

function NewHollandBB940A:brake()
   setVisibility(self.brakeLight, true);
end;

function NewHollandBB940A:releaseBrake()
    setVisibility(self.brakeLight, false);
end;